/* Author: J. Zobel, May 2001 */

/*
Build a continuous (well, kind of) cumulative distribution from a
discrete distribution.  In this case, it is for 27 characters, and is
with static arrays.
*/

#define NUMCHARS 27

static int charfreqs[NUMCHARS] = 
{
/*  a  b  c  d  e  f  g  h  i  j  k  l  m  n  o  p  q  r  s  t  u  v  w  x  y  z  sp */
    16,2, 4, 8, 16,4, 8, 4, 8, 2, 2, 8, 4, 8, 8, 4, 1, 8, 8, 8, 4, 2, 2, 1, 4, 1, 16
};

static double cmltcharfreqs[NUMCHARS+1];


/* Convert the original frequency distribution to a cumulative probability
distribution. */
void
makecmltfreqs()
{
    int		i;
    double	sum;
    
    for( i=0, sum=0.0 ; i<NUMCHARS ; i++ )
    {
	cmltcharfreqs[i] = sum;
	sum += (double) charfreqs[i];
    }

    cmltcharfreqs[i] = sum;

    for( i=0 ; i<=NUMCHARS ; i++ )
	cmltcharfreqs[i] /= sum;

    return;
}


/* The distribution cumulates to 1, now access it via a real
parameter over a range of 0 -> 1, interpolate intermediate values.
Interpolation allows the construction of the distribution function
to be more accurate (see distrand.c).
*/
double
discretedist(double v)
{
    int		idx;
    double	rem;

    v *= (double)NUMCHARS;
    idx = (int)v;
    rem = (double)v - idx;

    if( idx==NUMCHARS )
	return(cmltcharfreqs[NUMCHARS]);
    else
    {
	return( cmltcharfreqs[idx]
		+ rem * ( cmltcharfreqs[idx+1] - cmltcharfreqs[idx] ) );
    }
}
